#!/usr/bin/perl -w

use strict;
use warnings;

package FamilyTreeData;

my @parent_array = ("father", "mother");

sub new {
  my ( $classname) = @_;
  my $self = {
    people => {},  #hash of Person
    children => {}  #hash of childres's array
  };
  return bless $self, $classname;
}
#######################################################
# returns the no. children of this person
sub numChildren {
  my ($self, $person) = @_;
  if (defined $self->{children}{ $person->{id} }) {
    return ( scalar @{ $self->{children}{ $person->{id} } } );
  }
  else {
    return 0;
  }
}

sub node_defined {
  my ( $self, $id ) = @_;
  if( defined $self->{children}{ $id } ) {
    return 1;
  }
  else {
    return 0;
  }
}

sub get_father_full_name {
  my ( $self, $id ) = @_;
  if( defined $self->{people}{ $id }->{father} ) {
  	return $self->{people}{ $id }->{father}->{name}->full_name();
  }
  else {
    return "UNKNOWN";
  }
}
sub get_mother_full_name {
  my ( $self, $id ) = @_;
  if( defined $self->{people}{ $id }->{mother} ) {
  	return $self->{people}{ $id }->{mother}->{name}->full_name();
  }
  else {
    return "UNKNOWN";
  }
}
sub get_spouses {
  my ($self, $person) = @_;
  my $spouse_set = Set::Scalar->new;
  my $parent_1 = $parent_array[$person->{gender}];
  my $parent_2 = $parent_array[1 - $person->{gender}];
  foreach my $child (@{$self->{children}{$person->{id}}}) {
    $spouse_set->insert($child->{$parent_2})
      if($child->{$parent_1} == $person && defined $child->{$parent_2});
  }
  return $spouse_set;
}
sub get_peers {
  my ( $self, $person ) = @_;
  if (defined $person->{mother}) {
    return grep { (!defined $_->{father} && !defined $person->{father}) ||
                  ($_->{father} == $person->{father})} 
      @{$self->{children}{$person->{mother}->{id}}};
  }
  elsif (defined $person->{father}) {
    return grep {(!defined $_->{mother} && !defined $person->{mother}) ||
                 ($_->{mother} == $person->{mother})} 
      @{$self->{children}{$person->{father}->{id}}};
  }
  else {
    return ($person);
  }  
}
sub get_soft_peers {
  my ( $self, $person, $parent_type ) = @_;
  my $other_parent_type = ( $parent_type eq "mother" ) ? "father" : "mother";
  if ( defined $person->{$parent_type} ) {
    grep {(!defined $_->{$other_parent_type} && defined $person->{$other_parent_type}) ||
          (defined $_->{$other_parent_type} && !defined $person->{$other_parent_type}) ||
          (defined $_->{$other_parent_type} && defined $person->{$other_parent_type} && 
           $_->{$other_parent_type} != $person->{$other_parent_type}) } 
      @{$self->{children}{$person->{$parent_type}->{id}}};
  }
  else{
    return ();
  }
}



1;